"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var easingEquations = {
    easeOutSine: function (pos) { return Math.sin(pos * (Math.PI / 2)); },
    easeInOutSine: function (pos) { return -0.5 * (Math.cos(Math.PI * pos) - 1); },
    easeInOutQuint: function (pos) {
        if ((pos /= 0.5) < 1) {
            return 0.5 * Math.pow(pos, 5);
        }
        return 0.5 * (Math.pow(pos - 2, 5) + 2);
    },
};
var scrollKeys = {
    37: true,
    38: true,
    39: true,
    40: true,
    32: true,
    33: true,
    34: true,
    35: true,
    36: true,
};
function preventDefaultForScrollKeys(e) {
    if (scrollKeys[e.keyCode]) {
        preventDefault(e);
        return false;
    }
}
function preventDefault(e) {
    if (e === void 0) { e = window.event; }
    if (e.preventDefault) {
        e.preventDefault();
    }
    e.returnValue = false;
}
function disableScroll() {
    window.addEventListener('DOMMouseScroll', preventDefault, false);
    window.onwheel = preventDefault;
    window.onmousewheel = document.onmousewheel = preventDefault;
    window.ontouchmove = preventDefault;
    document.onkeydown = preventDefaultForScrollKeys;
}
exports.disableScroll = disableScroll;
function enableScroll() {
    window.removeEventListener('DOMMouseScroll', preventDefault, false);
    window.onmousewheel = document.onmousewheel = null;
    window.onwheel = null;
    window.ontouchmove = null;
    document.onkeydown = null;
}
exports.enableScroll = enableScroll;
function tweenValue(finalDelta, callback, speed, easing) {
    if (speed === void 0) { speed = 2000; }
    if (easing === void 0) { easing = 'easeInOutQuint'; }
    var startTime = window.performance.now();
    var time = Math.max(0.1, Math.min(Math.abs(finalDelta) / speed, 0.8)) * 1000;
    function tick(currentTime) {
        var elapsedTime = currentTime - startTime;
        var progress = elapsedTime / time;
        var t = easingEquations[easing](progress);
        if (progress < 1) {
            window.requestAnimationFrame(tick);
            callback(finalDelta * t);
        }
        else {
            callback(finalDelta);
        }
    }
    window.requestAnimationFrame(tick);
}
function scrollToY(endY, speed, easing, element) {
    if (endY === void 0) { endY = 0; }
    if (speed === void 0) { speed = 2000; }
    if (easing === void 0) { easing = 'easeInOutQuint'; }
    if (element === void 0) { element = window.document.scrollingElement; }
    var startY = element.scrollTop;
    function scrollMove(movement) {
        element.scrollTop = startY + movement;
    }
    tweenValue(endY - startY, scrollMove, speed, easing);
}
exports.scrollToY = scrollToY;
function getScrollTop() {
    return document.documentElement.scrollTop || document.body.scrollTop;
}
exports.getScrollTop = getScrollTop;
function getElementOffsetTop(element) {
    return element.getBoundingClientRect().top + getScrollTop();
}
exports.getElementOffsetTop = getElementOffsetTop;
